import { ImplementableApi } from './implementableApi';

import pino, { Logger } from 'pino';

namespace LogWriter {
    export type Config = ImplementableApi.Config & {
        path: string;
    };
}
/**
 * check nodejs buffer et throttle
 */
class LogWriter extends ImplementableApi {
    readonly path: string;
    readonly logger: Logger;

    constructor(readonly config: LogWriter.Config) {
        super(config);
        this.path = config.path;

        this.logger = pino(pino.destination({ dest: config.path }));
    }

    private writeMsg(message: string): void;
    private writeMsg(message: ImplementableApi.Message): void;
    private writeMsg(message: ImplementableApi.Message | string) {
        if (typeof message !== 'string')
            message = `[${message.type}] ${
                typeof message.rawContent === 'string'
                    ? message.rawContent
                    : JSON.stringify(message.rawContent)
            } ${
                message.idListener ??
                `( listener_id : ${message.idListener} )\n`
            }`;

        // const str = `[${new Date().toISOString()}] ${message}\n`;
        this.logger.info(message);
    }

    public receivedMessage(message: ImplementableApi.Message) {
        this.writeMsg(message);
    }
}

export { LogWriter };
