import { ImplementableApi } from './implementableApi';

import EventEmitter from 'events';

namespace Router {
    export type EventsType = {
        name: string;
        type: 'emit' | 'received';
    };
    export type InternalConfig = {
        events: EventsType[];
    };
}

class Router<
    MessageType extends ImplementableApi.TBaseMessageType
> extends EventEmitter {
    api_array: { [key: string]: ImplementableApi<MessageType> } = {};
    events: Router.EventsType[];

    constructor(readonly config: Router.EventsType[]) {
        super();
        this.events = config;
    }

    public injectDependency(api: ImplementableApi<MessageType>): void {
        if (api.name in this.api_array)
            throw `The api name '${api.name}' is already take`;
        this.api_array[api.name] = api;

        this.setEvents(api);

        this.receivedMessage({
            rawContent: `The dependency \`${api.name}\` was well injected into the router`,
            type: 'logging',
        });
    }

    private setEvents(api: ImplementableApi<MessageType>) {
        for (const event of this.events.filter(
            (ev) => ev.type === 'received'
        )) {
            api.on(event.name, (obj: ImplementableApi.Message<MessageType>) => {
                this.receivedMessage({
                    type: 'logging',
                    rawContent: `A message of type \`${obj.type}\` was emited by \`${api.name}\` with the event \`${event.name}\``,
                });
                this.emit(event.name, obj);
            });
        }
    }

    public receivedMessage(message: ImplementableApi.Message<MessageType>) {
        this.redirectMessage({
            rawContent: `A message of type \`${message.type}\` was received`,
            type: 'logging',
        });
        this.redirectMessage(message);
    }
    private redirectMessage(message: ImplementableApi.Message<MessageType>) {
        for (const api in this.api_array)
            this.api_array[api].receivedMessage(message);
    }
}

export { Router };
